@@ -5,7 +5,9 @@
  */
 #include <fenv.h>
 
+#if !(defined(_ARM_) || defined(__arm__))
 extern int __mingw_has_sse (void);
+#endif /* !(defined(_ARM_) || defined(__arm__)) */
 
 /* 7.6.2.4
    The fesetexceptflag function sets the complete status for those
@@ -21,6 +23,13 @@ int fesetexceptflag (const fexcept_t * flagp, int excepts)
   fenv_t _env;
 
   excepts &= FE_ALL_EXCEPT;
+
+#if defined(_ARM_) || defined(__arm__)
+  __asm__ volatile ("fmrx %0, FPSCR" : "=r" (_env));
+  _env.__cw &= ~excepts;
+  _env.__cw |= (*flagp & excepts);
+  __asm__ volatile ("fmxr FPSCR, %0" : : "r" (_env));
+#else
   __asm__ volatile ("fnstenv %0;" : "=m" (_env));
   _env.__status_word &= ~excepts;
   _env.__status_word |= (*flagp & excepts);
@@ -35,5 +44,6 @@ int fesetexceptflag (const fexcept_t * flagp, int excepts)
       __asm__ volatile ("ldmxcsr %0" : : "m" (sse_cw));
     }
 
+#endif /* defined(_ARM_) || defined(__arm__) */
   return 0;
 }
